
#ifndef __BUFFER_H_
#define __BUFFER_H_
#include "types.h"

/* Buffer type */
typedef struct buffer buffer_t;

/* Buffer configuration */
typedef struct
{
    uint8_t * data;
    uint16_t size;          /* NOTE: The REAL maximum size of the buffer will size-1 due to the round buffer nature */

} buffer_conf_t;

extern ret_code_t buffer_init( buffer_t ** __buffer, const buffer_conf_t * _conf );
extern ret_code_t buffer_init_shell( buffer_t ** __buffer );
extern ret_code_t buffer_config( buffer_t * _buffer, const buffer_conf_t * _conf );
extern void buffer_clear( buffer_t * _buffer );

extern void buffer_make_clone( buffer_t * _clone, const buffer_t * _image );

extern void buffer_set_freesize_limit( buffer_t * _buffer, uint16_t freesize_limit );
extern void buffer_reset_freesize_limit( buffer_t * _buffer );

extern buffer_conf_t buffer_get_conf( buffer_t * _buffer );
extern uint16_t buffer_get_read_pos( buffer_t * _buffer );
extern uint16_t buffer_get_write_pos( buffer_t * _buffer );
extern uint16_t buffer_get_loadsize( buffer_t * _buffer );
extern uint16_t buffer_get_freesize( buffer_t * _buffer );
extern uint16_t buffer_get_max_loadsize( buffer_t * _buffer );

extern void buffer_update_read_pos( buffer_t * _buffer, int16_t len );
extern void buffer_update_write_pos( buffer_t * _buffer, int16_t len );
extern void buffer_set_read_pos_safe( buffer_t * _buffer, uint16_t read_pos );
extern void buffer_set_write_pos_safe( buffer_t * _buffer, uint16_t write_pos );

extern ret_code_t buffer_move( buffer_t * _buffer_from, buffer_t * _buffer_to, uint16_t len );
extern ret_code_t buffer_copy( buffer_t * _buffer_from, uint16_t from_position, buffer_t * _buffer_to, uint16_t len );
extern ret_code_t buffer_move_any( buffer_t * _buffer_from, buffer_t * _buffer_to, uint16_t * len_moved );
extern void buffer_insert( buffer_t * _buffer, const uint8_t data[], uint16_t write_pos, uint16_t len );
extern ret_code_t buffer_add( buffer_t * _buffer, const uint8_t data[], uint16_t len );
extern ret_code_t buffer_get( buffer_t * _buffer, uint8_t * data, uint16_t len );
extern ret_code_t buffer_get_from_position( buffer_t * _buffer, uint16_t from_position, uint8_t * data, uint16_t len );
extern ret_code_t buffer_get_and_discard( buffer_t * _buffer, uint8_t * data, uint16_t len );

extern uint8_t * buffer_get_load_space_pointer( buffer_t * _buffer, uint16_t offset );
extern uint8_t * buffer_get_free_space_pointer( buffer_t * _buffer );
extern uint16_t buffer_get_load_space_line_size( const buffer_t * _buffer, uint16_t offset );
extern uint16_t buffer_get_free_space_line_size( const buffer_t * _buffer );

extern uint16_t buffer_calc_new_pos( const buffer_t * _buffer, uint16_t old_pos, int16_t offset );

extern void buffer_print_data( const buffer_t * _buffer );

#endif /* __BUFFER_H_ */
